#version 140

#ifdef GL_ES
precision highp float;
#endif

out vec4 fragColor;

// projected shadow texture
uniform sampler2D tex;
// deferred inputs
// deferred_normal
uniform sampler2D tex3;
// depth
uniform sampler2D tex5;

// xy = positive 1/screen dims
// zw = projection matrix components
uniform vec4 ofs;

// these two specify frustum corners
uniform vec3 corner;
uniform vec3 cornerDelta;

uniform mat4 shadow_projection_matrix;

// xy = offset, zw = scale
uniform vec4 uv_offset;

float LinearDepth(float d)
{
	// now with pre-baked denormalization and sign flip
	return ofs.w / (ofs.z + d);
}

void main()
{
	vec2 uv = gl_FragCoord.xy * ofs.xy;
	float storedDepth = texture(tex5, uv).x;

	// unpack world space normal
	vec3 normal = texture(tex3, uv).xyz;
	normal *= 2.0;
	normal -= vec3(1.0);
	// renormalization is necessary to reduce quantization errors
	normal = normalize(normal);

	vec3 viewPos = vec3(0.0);

	float linear_depth;
	
	linear_depth = LinearDepth(storedDepth);
	viewPos = (corner + gl_FragCoord.xyz*cornerDelta)*linear_depth;

	vec4 shadow_projected_uvs = shadow_projection_matrix * vec4(viewPos, 1.0);

	vec3 final;

	if (shadow_projected_uvs.z >= 0.0 && shadow_projected_uvs.z <= 1.0)
	{
		vec3 projuv = shadow_projected_uvs.xyz / shadow_projected_uvs.w;

		// check that projected uvs are within range
		projuv.xy = clamp(projuv.xy, vec2(0.0), vec2(1.0));

		projuv.xy *= uv_offset.zw;
		projuv.xy += uv_offset.xy;

		// this controls ground shadow amount
		final = vec3(texture(tex, projuv.xy).r*0.4 + 0.6);

		final = mix(final, vec3(1.0), clamp(projuv.z, 0.0, 1.0));

		if (normal.z <= 0.1)
			final = vec3(1.0);
	}
	else
		final = vec3(1.0);

	fragColor.xyz = final;
	fragColor.w = 0.0;
}
