#version 140

#include "include/constants.vert"
#include "include/clip_plane.vert"
#include "include/quat_vec.vert"

in vec3 pos;
in vec2 uv;
in vec2 uv2;
in vec4 color;
in vec4 norm;
in vec4 tang;

uniform vec4 model_quat[MAX_MODEL_INSTANCES];
// w = uniform scale
uniform vec4 model_trans[MAX_MODEL_INSTANCES];

uniform vec4 lightmap_offset[MAX_MODEL_INSTANCES];
uniform mat4 proj;

#ifdef NO_LIGHTMAP
uniform vec3 ambColor[MAX_MODEL_INSTANCES];
uniform vec3 lightColor[MAX_MODEL_INSTANCES];
uniform vec3 lightDir[MAX_MODEL_INSTANCES];
#endif

out vec2 uvOut;
out vec2 uvOut2;
out vec4 colOut;

out vec3 worldNormOut;

void main(void)
{
	vec3 vpos = pos;
	vec3 vnorm = quat_vec(model_quat[gl_InstanceID], norm.xyz);
	vec3 vtang = quat_vec(model_quat[gl_InstanceID], tang.xyz);

	// okay, so we have position, normal, tangent and bitangent

	vpos = quat_vec(model_quat[gl_InstanceID], vpos) * model_trans[gl_InstanceID].w +
		model_trans[gl_InstanceID].xyz;

	clip_vertex(vpos);
	vec3 vbitang = cross(vnorm, vtang) * tang.w;

	worldNormOut = vnorm*0.5 + vec3(0.5);

#ifdef NO_LIGHTMAP
	float ln = dot(vnorm, lightDir[gl_InstanceID]);

	// half-lambert
	ln = ln*0.5 + 0.5;
#endif

	gl_Position = proj * vec4( vpos, 1.0 );
	uvOut = uv;

	vec4 ofs = lightmap_offset[gl_InstanceID];

	uvOut2 = uv2*ofs.zw + ofs.xy;
	colOut = color;
#ifdef NO_LIGHTMAP
	colOut.xyz *= (lightColor[gl_InstanceID]*ln + ambColor[gl_InstanceID])*2.0;
#endif
}
