#version 140

// old-school lightmapping
#ifdef GL_ES
precision highp float;
#endif

#include "include/fragData.frag"
#include "include/clip_plane.frag"

//#define USE_GAMMA

// texture uvs
in vec2 uvOut;
// lightmap uvs
in vec2 uvOut2;

// diffuse color
uniform sampler2D tex;
// base color lightmap
uniform sampler2D tex2;
// interior_cubemap
uniform samplerCube tex3;

// interior tiling
uniform vec2 interior_tiling;

#ifdef INTERIOR_VARIANCE
uniform vec3 interior_variance_lo;
uniform vec3 interior_variance_hi;
#endif

in vec3 worldDirOut;

#ifndef NO_NORM_BUFFER
in vec3 worldNormOut;
#endif

#ifdef INTERIOR_VARIANCE
// FIXME: move elsewhere

// uv assumed to be float integers
float randf_intuv(vec2 uv, float seed)
{
	// inspired by https://www.shadertoy.com/view/4djSRW
	vec3 uvw = vec3(uv.xy, uv.x*uv.y + seed);

	vec3 tmp = fract(uvw * 0.5678);
	tmp += cross(tmp, vec3(1.23, -4.56, 7.89));

	return fract(tmp.x * tmp.y + tmp.z);
}
#endif

void main(void)
{
	clip_pixel();

/*	mat4 dither = mat4(
	0.0, 8.0, 2.0, 10.0,
	12.0, 4.0, 14.0, 6.0,
	3.0, 11.0, 1.0, 9.0,
	15.0, 7.0, 13.0, 5.0) / (16.0*128.0);*/

	vec4 b0 = unpack_lightmap(texture(tex2, uvOut2));

	// new: testing RGBM max 5 (ugly banding still but that's mostly because my rendertarget is 8 bit) + interpolation errors!
	//b0.xyz *= vec3(b0.w*6.0);

#ifdef USE_GAMMA
	// unpack lightmap
	b0.xyz = pow(b0.xyz, vec3(2.2));
#endif

	vec4 diffuse;
	vec4 final;

#ifndef SUPERSAMPLE_DIFFUSE
	diffuse = texture(tex, uvOut);
#else
	vec2 samples[8]=vec2[8](
	vec2(-0.973062, -0.328256),
	vec2(0.110481, 0.712614),
	vec2(-0.818603, 0.638871),
	vec2(-0.018864, -0.385902),
	vec2(-0.430052, 0.142593),
	vec2(0.601915, 0.342594),
	vec2(0.554622, -0.808192),
	vec2(-0.474124, -0.883385)
	);

	vec4 sum = vec4(0.0);

	vec2 ofs = vec2(1.0) / textureSize(tex, 0);

	for (int i=0; i<8; i++)
	{
		vec4 sample = texture2D(tex, uvOut + (samples[i])*ofs);
		sum += sample;
	}

	sum *= 1.0/8.0;

	diffuse = sum;

#endif

#ifdef USE_GAMMA
	diffuse.xyz = pow(diffuse.xyz, vec3(2.2));
#endif

	vec3 dir = worldDirOut;

	// reference: https://www.humus.name/index.php?page=3D&ID=80
	vec2 uvFinal = uvOut * interior_tiling.xy;
	vec2 fuv = fract(uvFinal);

#ifdef INTERIOR_VARIANCE
	float rnd = randf_intuv(floor(uvFinal), 0.34);
	vec3 interiorScale = mix(interior_variance_lo, interior_variance_hi, rnd);
#endif

	fuv *= 2.0;
	fuv -= vec2(1.0);

	// this works but for one direction only...
	vec3 pos = vec3(fuv, 1.0);

	vec3 id = vec3(1.0) / dir;
	vec3 k = abs(id) - pos*id;

	float kmin = min(min(k.x, k.y), k.z);
	pos += vec3(kmin) * dir;

	// v-flip final result
	pos.y *= -1.0;

	vec3 interior = texture(tex3, pos, 0.0).xyz;

#ifdef INTERIOR_VARIANCE
	interior *= interiorScale;
#endif

#ifndef INTERIOR_AFTER_LIGHTMAPS
	diffuse.xyz = mix(interior, diffuse.xyz, diffuse.w);
#endif


#ifndef NO_SPEC_BUFFER
	fragData3 = vec4(0.0);
#endif

#ifndef NO_NORM_BUFFER
	fragData2.xyz = worldNormOut;
#endif
	fragData1 = diffuse;

	// use diffuse alpha for potential transparency
	final.xyz = b0.xyz;

	// fake ambient
	//final.xyz = max(final.xyz, vec3(0.2));

	final.w = diffuse.w;

	// 2x overbright
	final.xyz *= diffuse.xyz * 2.0;

#ifdef INTERIOR_AFTER_LIGHTMAPS
	final.xyz = mix(interior, final.xyz, final.w);
#endif

#ifdef USE_GAMMA
	final.xyz = pow(final.xyz, vec3(1.0/2.2));
#endif

	fragColor = final;
}
