#version 140

#include "include/quat_vec.vert"
#include "include/clip_plane.vert"

in vec3 pos;
in vec2 uv;
in vec2 uv2;
in vec4 tang;
in vec3 norm;

uniform mat4 proj;
uniform vec4 model_trans;
uniform vec4 model_quat;

#ifdef UV_PAN
uniform float u_pan;
uniform float v_pan;
#endif

out vec2 uvOut;
out vec2 uvOut2;

out vec3 worldNormOut;

#if defined(REFRACT) || defined(MIRROR_FRESNEL)
uniform vec3 world_camera_position;
out vec3 worldDirOut;
#endif

out mat3 tang_to_world;

void main(void)
{
	vec3 vpos = pos;
	vpos = quat_vec(model_quat, vpos) * model_trans.w +
		model_trans.xyz;

	clip_vertex(vpos);

	gl_Position = proj * vec4(vpos, 1.0);

#ifdef UV_PAN
	uvOut = uv + vec2(u_pan, v_pan);
#else
	uvOut = uv;
#endif
	uvOut2 = uv2;

	vec3 normal = quat_vec(model_quat, norm);

	worldNormOut = normal * 0.5 + vec3(0.5);

	vec3 tangent = quat_vec(model_quat, tang.xyz);
	vec3 bitangent = cross(normal, tangent.xyz) * tang.w;

	// so: we have worldspace tangent...
	// we need to transform tangent to worldspace for normals
	tang_to_world[0] = tangent;
	tang_to_world[1] = bitangent;
	tang_to_world[2] = normal;

#if defined(REFRACT) || defined(MIRROR_FRESNEL)
	worldDirOut = vpos.xyz - world_camera_position;
#endif
}
