#version 140

// old-school lightmapping
#ifdef GL_ES
precision highp float;
#endif

#include "include/fragColor.frag"
#include "include/clip_plane.frag"

#ifdef USE_TRANSPARENT_LIGHTS
#include "include/transparent_lights.frag"
#endif

// texture uvs
in vec2 uvOut;
// lightmap uvs
in vec2 uvOut2;

// diffuse color
uniform sampler2D tex;
// base color lightmap
uniform sampler2D tex2;

// normal
uniform sampler2D tex3;

// cubemap
uniform samplerCube tex4;

// depth buffer texture
//uniform sampler2D tex5;

// fresnel at 0 deg
uniform float F0;
// dudv xy to normal variation
uniform float dudv_normal_mult;

uniform float animTime0;
uniform float animTime1;

uniform vec4 color;

uniform vec4 rcp_resolution;

// [0] = min rel, [1] = max, rel, [2] = relative_worldpos
uniform mat3 reflection_probe_box_relative;

in vec3 worldNormOut;
in vec3 worldDirOut;

#ifdef USE_TRANSPARENT_LIGHTS
in vec3 viewPosOut;
in vec3 viewNormOut;
#endif

// reference: https://tavianator.com/fast-branchless-raybounding-box-intersections-part-2-nans/
// note: ray starts at origin, only returns tmax
float RaycastBox(vec3 ray_dir, vec3 minv, vec3 maxv)
{
	vec3 invDir = vec3(1.0) / ray_dir;

	vec3 t1 = minv * invDir;
	vec3 t2 = maxv * invDir;
	vec3 tmax = max(t1, t2);

	return min(tmax[0], min(tmax[1], tmax[2]));
}

vec3 BoxCorrectVector(vec3 rp, vec3 relWorldPos)
{
	vec3 bmin = reflection_probe_box_relative[0];
	vec3 bmax = reflection_probe_box_relative[1];
	vec3 bcenter = reflection_probe_box_relative[2];

	bmin -= relWorldPos;
	bmax -= relWorldPos;

	float t = RaycastBox(rp, bmin, bmax);

	vec3 onBox = rp * t + relWorldPos;
	return onBox - bcenter;
}

void main(void)
{
	clip_pixel();

	vec4 b0 = unpack_lightmap(texture(tex2, uvOut2));

	//animTime1 = 0.001;

	vec4 dudv = texture(tex3, vec2(0.5 + uvOut.x + animTime1, uvOut.y + animTime1));
	dudv *= 2.0;
	dudv -= 1.0;

	vec4 dudv2 = texture(tex3, vec2(0.75+uvOut.x + animTime1, uvOut.y - animTime1));
	dudv2 *= 2.0;
	dudv2 -= 1.0;

	//dudv = max(dudv, dudv2);
	dudv *= dudv2;

	dudv *= 0.75;

	vec4 diffuse;
	vec4 final;

	diffuse = texture(tex, uvOut + dudv.xy);
	diffuse *= color;

	// use diffuse alpha for potential transparency
	final.xyz = b0.xyz;
	final.w = diffuse.w;

#ifdef USE_TRANSPARENT_LIGHTS
	// also, we don't normalize viewNormOut because of performance
	final.xyz += trans_compute_dynamic_lights(viewPosOut, gl_FrontFacing ? viewNormOut : -viewNormOut) * 0.5;
#endif

	// 2x overbright
	final.xyz *= diffuse.xyz * 2.0;

	vec3 norm = normalize(worldNormOut + vec3(dudv.x, dudv.y, 0.0)*dudv_normal_mult);

	vec3 dir = normalize(worldDirOut);

	vec3 rd = reflect(dir, norm);

	rd = BoxCorrectVector(rd, worldDirOut);

	// note: this hack will only work for xy-aligned water
	// this forces to sample from appropriate upper-or-lower part of the cubemap
	rd.z = abs(rd.z) * -sign(dir.z);

	// note: should be -r.z, but actually this is how my cubemap "screenshot" works
	vec3 reflectDir = vec3(-rd.y, rd.z, rd.x);

	vec4 reflColor = textureLod(tex4, reflectDir, 0.0);

	float NdotV = abs(dot(norm, dir));

	// Fresnel-Schlick
	float fresnel = F0 + (1.0-F0)*pow(max(0.0, 1.0 - NdotV), 5.0);

	// assuming reflColor has full alpha
	final = mix(final, reflColor, fresnel);

	fragColor = final;
}
