#version 140

#ifdef GL_ES
precision highp float;
#endif

out vec4 fragColor;

uniform sampler2D tex;

// this might be useful for color grading
//#define KEEP_LO_BITS

#ifdef USE_PALETTE
uniform sampler2D tex2;
uniform float palette_blend;
#endif

in vec2 uvOut;

uniform float gamma;
uniform float brightness;
uniform float contrast;
uniform float saturation;

#include "Tonemap.frag"

void main()
{
	fragColor.xyz = texture(tex, uvOut).xyz;

	fragColor.xyz = Tonemap(fragColor.xyz);
	fragColor.xyz = clamp(fragColor.xyz, vec3(0.0001), vec3(1.0));
	fragColor.xyz = linear_to_srgb(fragColor.xyz);

	// these need to be applied in sRGB otherwise it's way too strong
	float luma = dot(fragColor.xyz, vec3(0.3, 0.59, 0.11));
	fragColor.xyz = mix(vec3(luma), fragColor.xyz, saturation);
	fragColor.xyz = (fragColor.xyz - vec3(0.5))*contrast + vec3(0.5 + brightness);
	fragColor.xyz = pow(fragColor.xyz, vec3(gamma));

#ifdef USE_PALETTE
#ifdef USE_PALETTE_DITHERING
	// Intel HD4000 crashes if this is const... tnx pta
	mat4 dither = mat4(
	0.0/16.0, 8.0/16.0, 2.0/16.0, 10.0/16.0,
	12.0/16.0, 4.0/16.0, 14.0/16.0, 6.0/16.0,
	3.0/16.0, 11.0/16.0, 1.0/16.0, 9.0/16.0,
	15.0/16.0, 7.0/16.0, 13.0/16.0, 5.0/16.0);

	ivec2 iv = ivec2(mod(gl_FragCoord.xy, vec2(4.0)));
	// note: we artificially dither 2x more here
	float dith = (dither[iv.y][iv.x] - 0.5) * (1.0/31.0);

	fragColor.xyz += vec3(dith);
#endif

	vec3 inputColor = clamp(fragColor.xyz, 0.0, 1.0);
	fragColor.xyz = inputColor;
	// compute RGB666 color
	vec3 rgb666 = fragColor.xyz * 63.0;
	// round properly
	rgb666 = floor(rgb666 + 0.5);

	// compute lo/hi coords
	int loCoord = (int(rgb666.z) + int(rgb666.y)*64) & 511;
	int hiCoord = (int(rgb666.x)*8 + (int(rgb666.y) >> 3));

	// and finally lookup precomputed palette
	vec2 uv = vec2(float(loCoord)*(1.0/511.0), float(hiCoord)*(1.0/511.0));
	fragColor.xyz = mix(fragColor.xyz, texture(tex2, uv).xyz, palette_blend);

#ifdef KEEP_LO_BITS
	fragColor.xyz = floor(fragColor.xyz*64.0)*(1.0/64.0);
	fragColor.xyz += fract(inputColor.xyz*64.0)*(1.0/64.0);
#endif

#endif

	fragColor.w = 1.0;
}
